package cn.com.taiji.common.model;

import cn.com.taiji.common.enums.ErrorCodeEnum;
import lombok.Data;

/**
 * @author zzyx 2024/2/4
 */
@Data
public class Result<T> {
    private T data;
    private String msg;
    private Integer code;

    public int getCode() {
        return code;
    }

    public void setCode(int code) {
        this.code = code;
    }

    public String getMsg() {
        return msg;
    }

    public void setMsg(String msg) {
        this.msg = msg;
    }

    public T getData() {
        return data;
    }

    public void setData(T data) {
        this.data = data;
    }

    public static <T> Result<T> ok() {
        return ok(null);
    }

    public static <T> Result<T> ok(T data) {
        return init(data, ErrorCodeEnum.SUCCESS);
    }

    public static <T> Result<T> error() {
        return init(null, ErrorCodeEnum.INTERNAL_SERVER_ERROR);
    }

    public static <T> Result<T> error(int code, String message) {
        return initByCodeAndMessage(code, message);
    }

    public static <T> Result<T> error(ErrorCodeEnum codeEnum) {
        return init(null, codeEnum);
    }

    private static <T> Result<T> init(T data, ErrorCodeEnum codeEnum) {
        Result<T> result = new Result<>();
        result.setData(data);
        result.setMsg(codeEnum.getMessage());
        result.setCode(codeEnum.getCode());
        return result;
    }

    private static <T> Result<T> initByCodeAndMessage(int code, String message) {
        Result<T> result = new Result<>();
        result.setData(null);
        result.setCode(code);
        result.setMsg(message);
        return result;
    }
}
